import { html } from '@whatsup/jsx';

const createComponent = (tag, styles) => {
  const uid = generateUid();
  return rawProps => {
    const props = {};
    const style = {};
    const classNames = [];
    const children = rawProps.children !== undefined ? Array.isArray(rawProps.children) ? rawProps.children : [rawProps.children] : undefined;

    for (const [key, val] of Object.entries(rawProps)) {
      if (key === 'children') {
        continue;
      }

      if (key.startsWith('__')) {
        const property = '--' + key.slice(2);
        style[property] = val;
        continue;
      }

      if (key === 'style') {
        if (typeof val === 'object') {
          Object.assign(style, val);
        }

        continue;
      }

      if (key === 'className') {
        if (!!val) {
          classNames.push(val);
        }

        continue;
      }

      if (key in styles) {
        if (val) {
          classNames.push(styles[key]);
        }

        continue;
      }

      props[key] = val;
    }

    if (!!Object.keys(style).length) {
      props.style = style;
    }

    if (!!classNames.length) {
      props.className = classNames.join(' ');
    }

    return html(tag, uid, undefined, undefined, props, children);
  };
};

function generateUid() {
  return (~~(Math.random() * 1e8)).toString(16);
}

export { createComponent };
//# sourceMappingURL=factory.js.map
